<?php

declare (strict_types=1);
namespace Drupal\rfc9557;

use Drupal\Component\Datetime\DateTimePlus;

final class DateTimeParts {

  public function __construct(
    public string $year,
    public string $month = '',
    public string $day = '',
    public string $hour = '',
    public string $minute = '',
    public string $second = '',
    public string $fracsec = '',
    public string $offset = '',
    public string $offset_hour = '',
    public string $offset_minute = '',
  ) {
  }

  /**
   * Split a date string into parts.
   *
   * Similar to date_parse but without validation and with different offset
   * handling for later RFC9557 validation.
   *
   * @param string $date
   *   Date in (partial) RFC3339 like format.
   *
   * @return static
   */
  public static function createFromDateString(string $date): ?static {
    $date_matches = $time_matches = $fracsec_matches = $offset_matches = $extended_matches = [];
    if (!preg_match(
      pattern: ParseRfc9557::DATE_PATTERN,
      subject: $date,
      matches: $date_matches
    )) {
      return NULL;
    }
    preg_match(
      pattern: ParseRfc9557::TIME_PATTERN,
      subject: $date,
      matches: $time_matches
    );
    preg_match(
      pattern: ParseRfc9557::FRACSEC_PATTERN,
      subject: $date,
      matches: $fracsec_matches,
    );

    preg_match(
      pattern: ParseRfc9557::OFFSET_PATTERN,
      subject: $date,
      matches: $offset_matches
    );
    if (preg_match(
      pattern: ParseRfc9557::Z_PATTERN,
      subject: $date
    )) {
      $offset_matches = ['', '-', '00', '00'];
    }

    return new static(
      year: $date_matches[1],
      month: $date_matches[2] ?? '',
      day: $date_matches[3] ?? '',
      hour: $time_matches[1] ?? '',
      minute: $time_matches[2] ?? '',
      second: $time_matches[3] ?? '',
      fracsec: $fracsec_matches[1] ?? '',
      offset: $offset_matches[1] ?? '',
      offset_hour: $offset_matches[2] ?? '',
      offset_minute: $offset_matches[3] ?? '',
    );
  }

  /**
   * Get padded full datetime.
   *
   * If this is a reduced accuracy date first values will be defaulted.
   *
   * @return static
   *   A new DateTimeParts object with parts completed.
   */
  public function padded(): static {
    $padded_date = clone $this;
    $padded_date->month = $padded_date->month ?: '01';
    $padded_date->day = $padded_date->day ?: '01';
    $padded_date->hour = $padded_date->hour ?: '00';
    $padded_date->minute = $padded_date->minute ?: '00';
    $padded_date->second = $padded_date->second ?: '00';
    return $padded_date;
  }

  /**
   * Create an ISO8610/RFC3339 like datetime string.
   *
   * @return string
   */
  public function toString(): string {
    // @todo can replace DateTimePlus function here.
    // Decide if leading 0 padding should happen on class creation.
    $datetime = DateTimePlus::arrayToISO($this->toArray());
    if ($this->fracsec !== '') {
      $datetime .= '.' . $this->fracsec;
    }
    if ($this->offset !== '') {
      // Z notation for '-00:00' is preferred.
      // https://www.rfc-editor.org/rfc/rfc9557.html#section-2.3
      if ($this->offsetIsZ()) {
        $datetime .= 'Z';
      }
      else {
        $datetime .= $this->offset . $this->offset_hour . ':' . $this->offset_minute;
      }
    }
    return $datetime;
  }

  /**
   * Helper method to check if offset is Z.
   */
  public function offsetIsZ(): bool {
    return $this->offset === '-' && $this->offset_hour === '00' && $this->offset_minute === '00';
  }

  /**
   * Convert to a Drupal DateTimePlus like keyed array.
   */
  public function toArray(): array {
    return (array) $this;
  }

}
